import React, { useEffect, useState } from 'react';
import {
  Box,
  FormControl,
  FormLabel,
  Input,
  Button,
  InputLeftElement,
  InputGroup,
  HStack,
  Textarea,
  FormErrorMessage,
  InputLeftAddon,
} from '@chakra-ui/react';
import { Field, Form, Formik } from "formik";
import { FaEnvelope, FaFacebook, FaTwitter, FaLinkedinIn, FaInstagram } from 'react-icons/fa';
import GooglePlacesAutocomplete from 'react-google-places-autocomplete';
import Select from 'react-select';
import makeAnimated from 'react-select/animated';
import TimeRangePicker from '@wojtekmaj/react-timerange-picker';
import { createBusiness } from '../../helpers/business/';
import { getCategories } from '../../helpers/categories/';
import { useHistory } from 'react-router-dom';

const businessFormValues = {
  name: '',
  aboutU: '',
  address: '',
  phone: '',
  email: '',
  instagram: '',
  twitter: '',
  facebook: '',
  linkedIn: '',
  from: 0,
  to: 0,
  days: [],
  categories: [],
}

const gmapsapikey = process.env.REACT_APP_GOOGLE_MAPS_API_KEY;
const days = [
  { value: "monday", label: "Dilluns" },
  { value: "tuesday", label: "Dimarts" },
  { value: "wednesday", label: "Dimecres" },
  { value: "thursday", label: "Dijous" },
  { value: "friday", label: "Divendres" },
  { value: "saturday", label: "Dissabte" },
  { value: "sunday", label: "Diumenge" }
];

const BusinessRegisterForm = () => {
  const [descWords, setDescWords] = useState(0);
  const [value, setValue] = useState(null);
  const [categories, setCategories] = useState([]);
  const [selectedCategories, setSelectedCategories] = useState([])
  const [selectedDays, setSelectedDays] = useState([]);
  const [time, setTime] = useState(['9:00', '20:00']);
  const [description, setDescription] = useState('')
  const animatedComponents = makeAnimated();
  const history = useHistory();

  useEffect(() => {
    async function fetchData() {
      const categories = await getCategories();
      categories.forEach(category => {
        // eslint-disable-next-line
        delete Object.assign(category, { ['value']: category['id'] })['id'];
        // eslint-disable-next-line
        delete Object.assign(category, { ['label']: category['name'] })['name'];
      });
      setCategories(categories);
    }
    fetchData();
  }, []);

  const handleSelectedItemsChange = (selectedItems) => {
    setSelectedCategories(selectedItems);
  };

  const handleSelectedDays = (days) => {
    setSelectedDays(days);
  }

  const countWords = (e) => {
    const text = e.target.value;
    setDescWords(text.length);
    setDescription(text);
  }

  const validateEmail = (value) => {
    if (value && ! /^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*$/.test(value)) {
      return "Has d'introduïr una adreça d'email vàlida.";
    }
  }

  const validatePhone = (value) => {
    const str = value.toString().replace(/\s/g, '');
    if (value && str.length === 9 && /^[569]{1}[0-9]{9}$/.test(str)) {
      return "Has d'introduir un numero de telefon vàlid";
    }
  }

  const submitForm = async (values) => {
    console.log(selectedDays);
    const {
      name,
      phone,
      twitter,
      instagram,
      facebook,
      linkedIn,
      email,
    } = values;
    const payload = {
      name,
      description,
      place_id: value.value.place_id,
      contactInfo: {
        phone,
        email,
        twitter,
        facebook,
        linkedIn,
        instagram
      },
      timeTable: {
        from: time[0],
        to: time[1],
        days: selectedDays.map(el => el.value),
      },
      categories: selectedCategories.map(el => el.value),
    }

    const business = await createBusiness({ payload });

    if (business) {
      history.push({ pathname: '/businesses/register/2', state: { businessId: business.id } });
    }

  }

  return (
    <div>
      <Box my={8} textAlign='left'>
        <Formik
          initialValues={businessFormValues}
          onSubmit={submitForm}
        >
          {(props) => (
            <Form>
              <HStack mt={2}>
                <Field name="name">
                  {({ field, form }) => (
                    <FormControl isRequired isInvalid={form.errors.name} pr={8}>
                      <FormLabel>Com es diu el teu negoci?</FormLabel>
                      <InputGroup>
                        <Input {...field} id="name" type='text' placeholder='Nom del negoci' />
                      </InputGroup>
                      <FormErrorMessage>{form.errors.name}</FormErrorMessage>
                    </FormControl>
                  )}
                </Field>
                <Field name="categories">
                  {({ field, form }) => (
                    <FormControl isRequired isInvalid={form.errors.categories} pl={8}>
                      <FormLabel>Selecciona una o mes categories on situaries el teu negoci</FormLabel>
                      <Select
                        isMulti
                        name="categories"
                        options={categories}
                        value={selectedCategories}
                        closeMenuOnSelect={false}
                        components={animatedComponents}
                        onChange={handleSelectedItemsChange}
                      />
                      <FormErrorMessage>{form.errors.categories}</FormErrorMessage>
                    </FormControl>
                  )}
                </Field>
              </HStack>
              <HStack mt={2}>
                <Field name="days">
                  {({ field, form }) => (
                    <FormControl isInvalid={form.errors.days} pr={8}>
                      <FormLabel>Dies obert</FormLabel>
                      <Select
                        isMulti
                        name="days"
                        options={days}
                        value={selectedDays}
                        closeMenuOnSelect={false}
                        components={animatedComponents}
                        onChange={handleSelectedDays}
                      />
                      <FormErrorMessage>{form.errors.instagram}</FormErrorMessage>
                    </FormControl>
                  )}
                </Field>
                <Field name="range">
                  {({ field, form }) => (
                    <FormControl isInvalid={form.errors.days} pl={8}>
                      <FormLabel>Horari</FormLabel>
                      <TimeRangePicker
                        onChange={setTime}
                        value={time}
                      />
                      <FormErrorMessage>{form.errors.instagram}</FormErrorMessage>
                    </FormControl>
                  )}
                </Field>
              </HStack>
              <Field name="aboutUs">
                {({ field, form }) => (
                  <FormControl isRequired mt={2}>
                    <FormLabel>Escriu una breu descripció del teu negoci (que feu, productes, filosofia...) {descWords}/<b>500</b> paraules</FormLabel>
                    <Textarea
                      {...field}
                      id="aboutUs"
                      h={150}
                      onChange={countWords}
                    />
                  </FormControl>
                )}
              </Field>
              <Field name="address">
                {({ field, form }) => (
                  <FormControl isRequired isInvalid={form.errors.address} mt={2}>
                    <FormLabel>Introdueix la direcció del teu negoci</FormLabel>
                    <GooglePlacesAutocomplete
                      {...field}
                      apiKey={gmapsapikey}
                      apiOptions={{ language: 'ca', region: 'es' }}
                      selectProps={{
                        value,
                        onChange: setValue,
                      }}
                      id="address"
                    />
                    <FormErrorMessage>{form.errors.address}</FormErrorMessage>
                  </FormControl>
                )}
              </Field>
              <HStack mt={2}>
                <Field name="email" validate={validateEmail}>
                  {({ field, form }) => (
                    <FormControl isRequired isInvalid={form.errors.email} pr={8}>
                      <FormLabel>Adreça de correu del negoci</FormLabel>
                      <InputGroup>
                        <InputLeftElement
                          pointerEvents="none"
                          children={<FaEnvelope color="gray.300" />}
                        />
                        <Input {...field} id="email" type='email' placeholder='Adreça de correu' />
                      </InputGroup>
                      <FormErrorMessage>{form.errors.email}</FormErrorMessage>
                    </FormControl>
                  )}
                </Field>
                <Field name="phone" validate={validatePhone}>
                  {({ field, form }) => (
                    <FormControl isRequired isInvalid={form.errors.phone} pl={8}>
                      <FormLabel>Telefon de contacte</FormLabel>
                      <InputGroup>
                        <InputLeftAddon children="+34" />
                        <Input {...field} id="phone" type="text" placeholder='6XX XXX XXX' />
                      </InputGroup>
                      <FormErrorMessage>{form.errors.phone}</FormErrorMessage>
                    </FormControl>
                  )}
                </Field>
              </HStack>
              <HStack mt={2}>
                <Field name="instagram">
                  {({ field, form }) => (
                    <FormControl isInvalid={form.errors.instagram} pr={8}>
                      <FormLabel>Instagram</FormLabel>
                      <InputGroup>
                        <InputLeftElement
                          pointerEvents="none"
                          children={<FaInstagram color="gray.300" />}
                        />
                        <Input {...field} id="instagram" type='text' placeholder='https://www.' />
                      </InputGroup>
                      <FormErrorMessage>{form.errors.instagram}</FormErrorMessage>
                    </FormControl>
                  )}
                </Field>
                <Field name="facebook" >
                  {({ field, form }) => (
                    <FormControl isInvalid={form.errors.facebook} pl={8}>
                      <FormLabel>Facebook</FormLabel>
                      <InputGroup>
                        <InputLeftElement
                          pointerEvents="none"
                          children={<FaFacebook color="gray.300" />}
                        />
                        <Input {...field} id="facebook" type='text' placeholder='https://www.' />
                      </InputGroup>
                      <FormErrorMessage>{form.errors.facebook}</FormErrorMessage>
                    </FormControl>
                  )}
                </Field>
              </HStack>
              <HStack mt={2}>
                <Field name="twitter">
                  {({ field, form }) => (
                    <FormControl isInvalid={form.errors.twitter} pr={8}>
                      <FormLabel>Twitter</FormLabel>
                      <InputGroup>
                        <InputLeftElement
                          pointerEvents="none"
                          children={<FaTwitter color="gray.300" />}
                        />
                        <Input {...field} id="twitter" type='text' placeholder='https://www.' />
                      </InputGroup>
                      <FormErrorMessage>{form.errors.twitter}</FormErrorMessage>
                    </FormControl>
                  )}
                </Field>
                <Field name="linkedIn">
                  {({ field, form }) => (
                    <FormControl isInvalid={form.errors.linkedIn} pl={8}>
                      <FormLabel>LinkedIn</FormLabel>
                      <InputGroup>
                        <InputLeftElement
                          pointerEvents="none"
                          children={<FaLinkedinIn color="gray.300" />}
                        />
                        <Input {...field} id="linkedIn" type='text' placeholder='https://www.' />
                      </InputGroup>
                      <FormErrorMessage>{form.errors.linkedIn}</FormErrorMessage>
                    </FormControl>
                  )}
                </Field>
              </HStack>
              <Box
                align='center'
                justifyContent='space-evenly'
              >
                <Button
                  bgGradient="linear(to-r, red.500, yellow.500)"
                  color="white"
                  _hover={{
                    bgGradient: "linear(to-l, #7928CA,#FF0080)",
                  }}
                  width='full'
                  mt={4}
                  isLoading={props.isSubmitting}
                  type="submit"
                  w={200}
                >
                  Enviar
                </Button>
              </Box>
            </Form>
          )}
        </Formik>
      </Box>
    </div>
  )
};

export default BusinessRegisterForm;