import React, { useState } from "react";
import {
  Box,
  FormControl,
  FormLabel,
  Input,
  Button,
  InputLeftElement,
  InputRightElement,
  InputGroup,
  Link,
  Text,
  FormErrorMessage,
} from '@chakra-ui/react';

import { FaEnvelope, FaEye, FaEyeSlash, FaKey, FaUser } from 'react-icons/fa';
import { useHistory } from "react-router-dom";
import { Field, Form, Formik } from "formik";
import { useAuth } from './../../hooks/auth';


const RegisterForm = () => {
  const auth = useAuth();
  const history = useHistory();
  const [show, setShow] = useState(false)

  const showPassword = () => setShow(!show);

  const validateEmail = (value) => {
    if (!value) {
      return "L'adreça es obligatòria.";
    }
    if (! /^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*$/.test(value)) {
      return "Has d'introduïr una adreça d'email vàlida.";
    }
  }

  const validateName = (value) => {
    if (!value) {
      return "Has d'introduïr un nom.";
    }
  }

  const passwordsMatch = (password, rePassword) => {
    return password === rePassword;
  }

  const validatePassword = (value) => {
    if (!value) {
      return "Introdueix una contrasenya."
    }
    if (value.length < 8) {
      return "La contrasenya ha de tenir 8 caràcters com a mínim."
    }
  }

  const validateRePassword = (value) => {
    if (!value) {
      return "Torna a introduïr la contrasenya."
    }
    if (value.length < 8) {
      return "La contrasenya ha de tenir 8 caràcters com a mínim."
    }
  }

  const submitRegisterForm = async (values, actions) => {
    if (!passwordsMatch(values.password, values.rePassword)) {
      actions.setSubmitting(false);
      actions.validateField('password')
    }

    auth.register({
      name: values.name,
      email: values.email,
      password: values.password
    }).then((res) => {
      actions.setSubmitting(false);
      history.push("/");
    })

  };

  return (
    <Box my={8} textAlign='left'>
      <Formik
        initialValues={{ name: "", email: "", password: "", rePassword: "" }}
        onSubmit={submitRegisterForm}
      >
        {(props) => (
          <Form>
            <Field name="name" validate={validateName}>
              {({ field, form }) => (
                <FormControl isRequired isInvalid={form.errors.name} mt={2}>
                  <FormLabel>Com et dius?</FormLabel>
                  <InputGroup>
                    <InputLeftElement
                      pointerEvents="none"
                      children={<FaUser color="gray.300" />}
                    />
                    <Input {...field} id="name" type='text' placeholder='Introdueix el teu nom' />
                  </InputGroup>
                  <FormErrorMessage>{form.errors.name}</FormErrorMessage>
                </FormControl>
              )}
            </Field>

            <Field name="email" validate={validateEmail}>
              {({ field, form }) => (
                <FormControl isRequired isInvalid={form.errors.email} mt={2}>
                  <FormLabel>Adreça de correu</FormLabel>
                  <InputGroup>
                    <InputLeftElement
                      pointerEvents="none"
                      children={<FaEnvelope color="gray.300" />}
                    />
                    <Input {...field} id="email" type='email' placeholder='Adreça de correu' />
                  </InputGroup>
                  <FormErrorMessage>{form.errors.email}</FormErrorMessage>
                </FormControl>
              )}
            </Field>

            <Field name="password" validate={validatePassword}>
              {({ field, form }) => (
                <FormControl isRequired isInvalid={form.errors.password} mt={2}>
                  <FormLabel>Contrasenya</FormLabel>
                  <InputGroup>
                    <InputLeftElement
                      pointerEvents="none"
                      children={<FaKey color="gray.300" />}
                    />
                    <Input
                      pr="4.5rem"
                      type={show ? "text" : "password"}
                      placeholder="Contrasenya"
                      {...field}
                    />
                    <InputRightElement
                      width="4.5rem"
                      children={show ? <FaEyeSlash color="gray.300" /> : <FaEye color="gray.300" />}
                      onClick={showPassword}
                    />
                  </InputGroup>
                  <FormErrorMessage>{form.errors.password}</FormErrorMessage>
                </FormControl>
              )}
            </Field>

            <Field name="rePassword" validate={validateRePassword}>
              {({ field, form }) => (
                <FormControl isRequired isInvalid={form.errors.rePassword} mt={2}>
                  <FormLabel>Repeteix la contrasenya</FormLabel>
                  <InputGroup>
                    <InputLeftElement
                      pointerEvents="none"
                      children={<FaKey color="gray.300" />}
                    />
                    <Input
                      pr="4.5rem"
                      type={show ? "text" : "password"}
                      placeholder="Repeteix la contrasenya"
                      {...field}
                    />
                    <InputRightElement
                      width="4.5rem"
                      children={show ? <FaEyeSlash color="gray.300" /> : <FaEye color="gray.300" />}
                      onClick={showPassword}
                    />
                  </InputGroup>
                  <FormErrorMessage>{form.errors.rePassword}</FormErrorMessage>
                </FormControl>
              )}
            </Field>


            <Text mt={4} fontSize="sm">La contrasenya ha de tenir més de 8 caràcters. </Text>
            <Button
              bgGradient="linear(to-r, red.500, yellow.500)"
              color="white"
              _hover={{
                bgGradient: "linear(to-l, #7928CA,#FF0080)",
              }}
              width='full'
              mt={4}
              isLoading={props.isSubmitting}
              type="submit"
            >
              Registar-me
            </Button>
            <Box align="center" justifyContent='center' mt={4}>
              <Link onClick={() => history.push("/login")} colorScheme={'purple'}>Ja estàs registrat? Entra!</Link>
            </Box>
          </Form>
        )}

      </Formik>
    </Box>
  )
}

export default RegisterForm;