const httpStatus = require('http-status');
const pick = require('../utils/pick');
const ApiError = require('../utils/ApiError');
const catchAsync = require('../utils/catchAsync');
const { businessService } = require('../services');
const { geolocatePlace, distanceMatrix, businessRating } = require('../utils/geolocator');
const { format } = require('util');

const createBusiness = catchAsync(async (req, res) => {
  req.body.address = await geolocatePlace(req.body.place_id);
  req.body.user = req.user.id;
  req.body.rating = await businessRating(req.body.place_id);
  const business = await businessService.createBusiness(req.body);
  res.status(httpStatus.CREATED).send(business);
});

const getBusinesses = catchAsync(async (req, res) => {
  const filter = pick(req.query, ['name', 'description']);
  const options = pick(req.query, ['sortBy', 'limit', 'page']);
  const result = await businessService.queryBusiness(filter, options);
  res.send(result);
});

const getBusiness = catchAsync(async (req, res) => {
  const business = await businessService.getBusinessById(req.params.businessId);
  if (!business) {
    throw new ApiError(httpStatus.NOT_FOUND, 'Business not found');
  }
  res.send(business);
});

const getBussinessByUserId = catchAsync(async (req, res) => {
  const business = await businessService.getBussinessByUserId(req.params.userId);
  if (!business) {
    throw new ApiError(httpStatus.NOT_FOUND, 'Business not found');
  }
  res.send(business);
});

const updateBusiness = catchAsync(async (req, res) => {
  const business = await businessService.updateBusinessById(req.params.businessId, req.body);
  res.send(business);
});

const deleteBusiness = catchAsync(async (req, res) => {
  await businessService.deleteBusinessById(req.params.businessId);
  res.status(httpStatus.NO_CONTENT).send();
});

const businessDistance = catchAsync(async (req, res) => {
  const distance = await distanceMatrix(req.body);
  res.send(distance);
});

const uploadImages = catchAsync(async (req, res) => {
  const { Storage } = require('@google-cloud/storage');

  const storage = new Storage({ keyFilename: 'nearly-gcloud.json' });
  const { businessId } = req.params;
  const { files } = req;

  const bucketName = process.env.BUCKET_IMAGES;
  const bucket = storage.bucket(bucketName);
  const gallery = [];
  for (let i = 0; i < files.length; i++) {
    const file = files[i];
    const newFileName = businessId + '/' + file.originalname.replace(/ /g, "_");
    console.log(newFileName);

    const fileUpload = bucket.file(newFileName);
    const blobStream = fileUpload.createWriteStream({
      metadata: {
        contentType: file.mimetype
      }
    });

    blobStream.on('finish', async () => {
      //await fileUpload.makePublic();
      // The public URL can be used to directly access the file via HTTP.
      const publicUrl = format(
        `https://storage.googleapis.com/${bucket.name}/${fileUpload.name}`
      );
      gallery.push({ url: publicUrl, name: newFileName });

      if (i === files.length - 1) {
        const business = await businessService.createBusinessImages({ businessId, gallery });
        res.status(201).json(business);
      }
    });

    blobStream.end(file.buffer);
  }
});

module.exports = {
  createBusiness,
  getBusiness,
  getBusinesses,
  updateBusiness,
  deleteBusiness,
  uploadImages,
  getBussinessByUserId,
  businessDistance,
};
